package org.gcube.dataanalysis.storagehubfacility;

import java.io.BufferedInputStream;
import java.net.HttpURLConnection;
import java.net.URL;

import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamConstants;
import javax.xml.stream.XMLStreamReader;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 
 * @author Giancarlo Panichi
 *
 */
public class ISSupport {
	private static Logger logger = LoggerFactory.getLogger(ISSupport.class);
	// Dev
	//private static final String IS_URL = "https://node10-d-d4s.d4science.org";
	// Prod
	private static final String IS_URL = "http://registry.d4science.org";
	private static final String storageHubServiceClass = "DataAccess";
	private static final String storageHubServiceName = "StorageHub";

	public ISSupport() {

	}

	public String discoverStorageHub(String gcubeToken) throws Exception {

		try {
			logger.debug("Discover StorageHub");

			String storageHubUrl = null;

			logger.debug("Retrieve IS info for StorageHub: " + IS_URL + "/icproxy/gcube/service/GCoreEndpoint/"
					+ storageHubServiceClass + "/" + storageHubServiceName + "?");

			String urlString = IS_URL + "/icproxy/gcube/service/GCoreEndpoint/" + storageHubServiceClass + "/"
					+ storageHubServiceName + "?gcube-token=" + gcubeToken;

			URL url = new URL(urlString);
			HttpURLConnection conn = (HttpURLConnection) url.openConnection();
			conn.setRequestMethod("GET");
			conn.setRequestProperty("Content-Type", "text/xml; charset=UTF-8");
			try (BufferedInputStream in = new BufferedInputStream(conn.getInputStream())) {
				XMLInputFactory xmlInFact = XMLInputFactory.newInstance();
				XMLStreamReader reader = xmlInFact.createXMLStreamReader(in);
				while (reader.hasNext()) {
					int eventType = reader.next();
					if (eventType == XMLStreamConstants.START_ELEMENT) {
						logger.debug("Read Element: " + reader.getLocalName());

						if (reader.getLocalName().compareToIgnoreCase("EndPoint") == 0) {
							logger.debug("Attribute count:" + reader.getAttributeCount());
							for (int i = 0; i < reader.getAttributeCount(); i++) {
								logger.debug("Attribute Name: " + reader.getAttributeName(i));
								logger.debug("Attribute Value: " + reader.getAttributeValue(i));
								if (reader.getAttributeName(i).toString().compareToIgnoreCase("EntryName") == 0
										&& reader.getAttributeValue(i).toString().compareToIgnoreCase(
												"org.gcube.data.access.storagehub.StorageHub") == 0) {
									storageHubUrl = reader.getElementText();
									logger.debug("StorageHub url retrieved: " + storageHubUrl);
									return storageHubUrl;
								}

							}

						}

					}

				}
			}
			logger.debug("StorageHub url retrieved: " + storageHubUrl);
			return storageHubUrl;

		} catch (Exception e) {
			logger.error("Error retrieving StorageHub url from IS: " + e.getLocalizedMessage(), e);
			throw e;
		}

	}

	public static void main(String[] args) {
		ISSupport isSupport = new ISSupport();
		try {
			isSupport.discoverStorageHub("gcube-token");
		} catch (Exception e) {

		}
	}

}
